<?php

if (! class_exists('EG_Series_data')) {

	/**
	 * Class EG_Series_data
	 *
	 * Implement a shortcode to display the posts series
	 *
	 */
	class EG_Series_Data {

		var $textdomain;
		var $series_list;
		var $posts_list;

		/**
		 * EG_Series_data
		 *
		 * Constructor for PHP 4
		 *
		 * @package EG-Series
		 * @param string	$textdomain	text domain of the plugin
		 * @return none
		 */
		function EG_Series_data($textdomain, $is_singleton=FALSE) {
			$is_singleton || die('Cannot instantiate '.get_class($this).' class directly!\n');

			register_shutdown_function(array(&$this, '__destruct'));
			$this->textdomain = $textdomain;
		} /* End of EG_Series_data */

		/**
		 * __construct
		 *
		 * Constructor for PHP 5
		 *
		 * @package EG-Series
		 * @param string	$textdomain	text domain of the plugin
		 * @return none
		 */
		function __construct($textdomain, $is_singleton=FALSE) {
			$is_singleton || die('Cannot instantiate '.get_class($this).' class directly!\n');

			$this->textdomain = $textdomain;
		} /* End of __construct */

		/**
		 * __destruct(constructor)
		 *
		 * @package EG-Forms
		 *
		 * @param	none
		 * @return 	none
		 */
		function __destruct() {
			$this->delete_lists();
		} /* End of __destruct */

		function &get_instance($textdomain) {
			static $eg_series_instance;

			if (!is_object($eg_series_instance))
				$eg_series_instance = new EG_Series_Data($textdomain, TRUE);

			return $eg_series_instance;
		} /* End of get_instance */


		/**
		 * delete_lists
		 *
		 * Clear $posts_list et $series_list
		 *
		 * @package EG-Series
		 * @param none
		 * @return none
		 */
		function delete_lists() {
			if (isset($this->posts_list)) {
				unset($this->posts_list);
				$this->posts_list = NULL;
			}
			if (isset($this->series_list)) {
				unset($this->series_list);
				$this->series_list = NULL;
			}
		} /* End of delete_lists */

		/**
		 * build_lists
		 *
		 * Build $posts_list and $series_list
		 *
		 * @package EG-Series
		 * @param string		$meta_key		meta key used to store serie
		 * @param boolean	$force_refresh	if TRUE, build lists even if they exist
		 * @return none
		 */
		function build_lists($meta_key, $force_refresh=FALSE) {
			global $wpdb;

			if ( ($meta_key != '' && !isset($this->series_list)) || $force_refresh) {

				// Delete lists before re-create them
				$this->delete_lists();

				// Get post and series from the database
				$results = $wpdb->get_results('SELECT DISTINCT ID, meta_value, post_title, post_date, post_status FROM '.$wpdb->posts.' , '.$wpdb->postmeta.' WHERE meta_key="'.$meta_key.'" AND ID=post_id ORDER BY meta_value ASC, post_date ASC');
// post_type="post" AND 
				if (isset($results)) {
					// List the results and build the series_list and the post_list
					foreach ($results as $result) {
						$this->posts_list[$result->ID] = $result->meta_value;
						$this->series_list[$result->meta_value][$result->ID] = array(
							'post_title'  => $result->post_title,
							'post_date'   => $result->post_date,
							'post_status' => $result->post_status
						);
					}
				}
			}
		} /* End of build_lists */

		/**
		 * refresh_list
		 *
		 * Re-build the lists evenif they already exist
		 *
		 * @package EG-Series
		 * @param string		$meta_key		meta key used to store serie
		 * @return none
		 */
		function refresh_list($meta_key) {
			$this->build_lists($meta_key, TRUE);
		} /* End of refresh_list */


		/**
		 * get_series_number
		 *
		 * Get number of series
		 *
		 * @package EG-Series
		 *
		 * @param none
		 * @return int		Number of series
		 */
		function get_series_number() {

			$number = 0;
			if (isset($this->series_list)) {
				$number = sizeof($this->series_list);
			}
			return ($number);
		}

		/**
		 * get_series_details
		 *
		 * Get the entire $series_list
		 *
		 * @package EG-Series
		 * @param string	$meta_key		meta key used to store serie
		 * @return array				series list
		 */
		function get_series_details($meta_key) {
			// Build the list if required
			$this->build_lists($meta_key);

			// Return list
			return ($this->series_list);
		} /* End of get_series_details */

		/**
		 * get_post_serie
		 *
		 * Get the serie of the specified post
		 *
		 * @package EG-Series
		 * @param string	$meta_key		meta key used to store serie
		 * @param int	$post_id		ID of the post
		 * @return string				serie of the post (or '')
		 */
		function get_post_serie($meta_key, $post_id) {
			// Build the list if required
			$this->build_lists($meta_key);

			if (isset($this->posts_list[$post_id])) return ($this->posts_list[$post_id]);
			else return ('');
		} /* get_post_serie */

		/**
		 * get_series
		 *
		 * Get series with different formats
		 *
		 * @package EG-Series
		 * @param string	$meta_key		meta key used to store serie
		 * @param string	$list_type		type of list (select, ol, ul, array)
		 * @param int		$width			width of the list
		 * @param int		$height			if select => height of the select field, if ol or ul => number of series to display
		 * @param string	$list_style		style to apply to the list
		 * @param boolean	$auto_submit	if select => submit field without click on button
		 * @param int		$more_series	id of a post or a page where the entire list of series can be displayed
		 * @param boolean	$add_no_serie	add additional empty option
		 * @param int		$post_id		ID of the post
		 * @return string					serie of the post (or '')
		 */
		function get_series($meta_key,
						$list_type    = 'ol',
						$width        = 0,
						$height       = 0,
						$title        = '',
						$titletag     = '',
						$select_name  = 'series_list',
						$default_value = '',
						$auto_submit  = FALSE,
						$more_series  = 0,
						$add_no_serie = FALSE) {

			// Build the lists if they are empty
			$this->build_lists($meta_key);

			$value = FALSE;
			// If list not empty
			if (isset($this->series_list) && sizeof($this->series_list)> 0) {

				// if ($list_style!='') $list_style = 'class="'.$list_style.'"';

				if ($title != '' && $titletag != '') $title_string = '<'.$titletag.'>'.htmlspecialchars($title).'</'.$titletag.'>';
				else $title_string = '';

				switch ($list_type) {
					case 'select':
						if (! $height) $height = 5;

						// Manage Auto-Submit
						if ($auto_submit) $onchange_string =  'onChange="window.location=this.value"';
						else $onchange_string = '';

						// Manage add_no_serie
						if ($add_no_serie) $value = '<option class="eg-series-series-item" value="no_serie"> </option>';

						// Build the list
						foreach ($this->series_list as $serie_name => $posts_list) {

							$first_post_id = key($posts_list);
							if ($width>0 && strlen($serie_name)>$width) $label = wp_html_excerpt( $serie_name, $width-3).'...';
							else $label = $serie_name;

							$selected = '';
							if ($auto_submit) $link = get_permalink($first_post_id);
							else {
								$link = $serie_name;
								if ($link == $default_value) $selected = 'selected';
							}

							$value .= '<option class="eg-series-series-item" value="'.$link.'" '.$selected.'>'.$label.'</option>';
						}
						// If list is not empty, finalize it
						if ($value != '') {
							$value = $title_string.'<select class="eg-series-series" id="'.$select_name.'" name="'.$select_name.'" size="'.$height.'" '.$onchange_string.'>'.$value.'</select><br />';
						}
					break;

					// HTML ul or ol list
					case 'ul':
					case 'ol':
						if (! $height) $height = 0;

						$number = 1;
						foreach ($this->series_list as $serie_name => $posts_list) {

							if ($width>0 && strlen($serie_name)>$width) $label = wp_html_excerpt( $serie_name, $width-3 ).'...';
							else $label = $serie_name;

							$first_post_id    = key($posts_list);
							$first_post_title = current($posts_list);
							$value .= '<li class="eg-series-series-item"><a href="'.get_permalink($first_post_id).'" title="'.$first_post_title.'">'.$label.'</a></li>';
							$number++;
							// Stop the list if we reach the number of series required
							if ($height!=0 && $number>$height) break;
						}
						if ($value != '') {
							// Add the "more series" link
							if ($more_series && $height!=0 && sizeof($series_list)>$height) {
								$value .= '<li class="eg-series-series-item"><a href="'.get_permalink($more_series).'" title="'.__('See more series', $this->textdomain).'">'.__('More series', $this->textdomain).'</a></li>';
							}
							// Finalize the list
							$value = $title_string.'<'.$list_type.' class="eg-series-series">'.$value.'</'.$list_type.'>';
						}
					break;

					case 'array':
						// Special type: not HTML code, just build an array
						if ($add_no_serie) $value = array( 'no_serie' => ' ');
						foreach ($this->series_list as $serie_name => $serie_values) {
							$value[$serie_name] = $serie_name;
						}
					break;
				} /* end of switch */
			} /* if sizeof ($series_list) ... */
			return ($value);
		} /* End of get_series */

		
		/**
		 * sort_posts
		 *
		 * Sort list of posts,
		 *
		 * @package EG-Series
		 *
		 * @param array		$posts_list			List of posts to sort,
		 * @param string	$orderby			sort key,
		 * @param string	$order				sort order
		 * @return array	 					sorted list of posts,
		 */
		function sort_posts($posts_list, $orderby, $order) {
		
			if ( strcasecmp($orderby,'date')==0 && strcasecmp($order,'asc')==0 ) {
				return ($posts_list);
			}
			else {
				$order = ( strcasecmp($order,'ASC')==0 ? -1 : 1 );
				if ( stripos($orderby, 'post_') === FALSE ) $orderby = 'post_'.$orderby;
				$result_code = uasort($posts_list, create_function('$a,$b', '$a1=$a['.$orderby.']; $b1=$b['.$orderby.']; if ($a1==$b1) return 0; else return ($a1<$b1) ? '.$order.' : '.-$order.';'));
				return ($posts_list);
			}			
		}
		
		/**
		 * Get_posts
		 *
		 * Get series with different formats
		 *
		 * @package EG-Series
		 * @param string	$meta_key			meta key used to store serie,
		 * @param string	$name				name of the serie,
		 * @param int		$current_post_id	ID of the post where to get serie,
		 * @param string	$list_type			list type (select, ol, ul),
		 * @param string	$title				title of the list,
		 * @param string	$titletag			html tag to use to display title,
		 * @param string	$order				post order (ASC or DESC)
		 * @return string						list of posts of the specified serie
		 */
		// function get_posts($meta_key='', $name, $current_post_id, $list_type, $title='', $titletag, $order) {
		function get_posts($meta_key,
							$name		  = '',
							$current_post_id,
							$list_type	  = 'ol',
							$width		  = 0,
							$height		  = 0,
							$title		  = '',
							$titletag	  = 'h2',
							$auto_submit  = TRUE,
							$orderby      = 'date',
							$order		  = 'DESC',
							$show_date    = 0,
							$date_format  = '') {
			global $wp_locale;

			// Build the list if required
			$this->build_lists($meta_key);

			$string = '';
			if (isset($this->series_list) && sizeof($this->series_list) > 0) {

				$serie = '';
				// Name specified
				if ($name != '' && isset($this->series_list[$name])) {
					$serie = $name;
				}
				elseif ($current_post_id != '' && $current_post_id != 0) {
					// if post exists and has a serie
					if (isset($this->posts_list[$current_post_id])) $serie = $this->posts_list[$current_post_id];
				}

				if ($serie == '')
					return ('');

				if ($title != '' && $titletag != '') $title_string = '<'.$titletag.'>'.htmlspecialchars($title).'</'.$titletag.'>';
				else $title_string = '';

				$post_list = $this->sort_posts($this->series_list[$serie], $orderby, $order);
				
				switch ($list_type) {

					case 'select':
						if ($auto_submit) $onchange_string = 'onChange="if (this.value!=\'\') { window.location=this.value}" ';
						else $onchange_string = '';

						if (! $height) $height = 5;

						foreach ($post_list as $post_id => $post_values) {

							$post_datetime = strtotime($post_values['post_date']);
							//$date_string = ' - '.date('d',$post_datetime).' '.$wp_locale->get_month_abbrev($wp_locale->get_month(date('n',$post_datetime)));
							if ($show_date) $date_string = ' '.date_i18n($date_format, $post_datetime);
							else $date_string = '';

							$this_post = '';
							if ($post_id == $current_post_id) {
								$this_post = __('(This post)', $this->textdomain);
								$width     = $width - strlen($this_post);
							}

							if ($width>0 && strlen($post_values['post_title'])>$width) 
								$label = wp_html_excerpt( $post_values['post_title'], $width-3).'...';
							else $label = $post_values['post_title'];
							
							// $label = htmlspecialchars($label);

							if ($post_id == $current_post_id) {				
								$string_to_add = '<option class="eg-series-posts-item current" value="#" selected>'.htmlspecialchars($label.' '.$this_post).'</option>';
							}
							else {
								if ($post_values['post_status'] == 'publish') {
									$string_to_add = '<option class="eg-series-posts-item" value="'.get_permalink($post_id).'" title="'.htmlspecialchars($post_values['post_title']).'">'.htmlspecialchars($label.' '.$date_string).'</option>';
								}
								else {
									$string_to_add = '<option class="eg-series-posts-item" value="">'.htmlspecialchars($label.$date_string).'</option>';
								}
							}
							$string .= $string_to_add;
						}
						// If list is not empty, finalize it
						if ($string  != '') {
							$string  = '<select '.$list_style.' class="eg-series-posts" id="series_list" name="series_list" size="'.$height.'" '.$onchange_string.'>'.$string .'</select><br />';
						}
					break;

					case 'ul':
					case 'ol':
						foreach ($post_list as $post_id => $post_values) {
							$post_datetime = strtotime($post_values['post_date']);
							if ($show_date) $date_string   = ' - <span class="date">'.date_i18n($date_format, $post_datetime).'</span>';
							else $date_string = '';

							$this_post = '';
							if ($post_id == $current_post_id) {
								$this_post = __('(This post)', $this->textdomain);
								$width     = $width - strlen($this_post);
							}
							
							if ($width>0 && strlen($post_values['post_title'])>$width) 
								$label = wp_html_excerpt( $post_values['post_title'], $width-3).'...';
							else $label = $post_values['post_title'];

							// $label = htmlspecialchars($label);

							if ($post_id == $current_post_id) {
								$string_to_add = '<li class="eg-series-posts-item current">'.htmlspecialchars($label.' '.$this_post).'</li>';
							}
							else {
								if ($post_values['post_status'] == 'publish') {
									$string_to_add = '<li class="eg-series-posts-item"><a href="'.get_permalink($post_id).'" title="'.htmlspecialchars($post_values['post_title']).'">'.htmlspecialchars($label).'</a> '.$date_string.'</li>';
								}
								else {
									$string_to_add = '<li class="eg-series-posts-item">'.htmlspecialchars($label.$date_string).'</li>';
								}
							}
							$string .= $string_to_add;
						}
						if ($string != '') $string = '<'.$list_type.' class="eg-series-posts">'.$string.'</'.$list_type.'>';
					break;
				}
			}
			if ($string != '' && $title!='') $string = '<'.$titletag.'>'.__($title, $this->textdomain).'</'.$titletag.'>'.$string;

			return ($string);
		} /* End of get_posts */
	}
}

?>